package org.eco.als.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import cn.hutool.core.util.ObjectUtil;
import jakarta.annotation.Resource;
import lombok.RequiredArgsConstructor;
import org.eco.als.domain.FalseAlarm;
import org.eco.als.domain.bo.FalseAlarmBo;
import org.eco.als.domain.vo.FalseAlarmVo;
import org.eco.als.service.IFalseAlarmService;
import org.eco.common.core.core.domain.CommonResult;
import org.eco.common.core.core.domain.model.LoginUser;
import org.eco.common.core.core.page.PageResult;
import org.eco.common.log.annotation.Log;
import org.eco.common.log.enums.BusinessType;
import org.eco.common.security.utils.LoginHelper;
import org.eco.common.web.annotation.RepeatSubmit;
import org.eco.common.web.core.BaseController;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * 虚警抑制Controller
 *
 * @author wgk
 * @date 2024-07-23
 */
@Validated
@RequiredArgsConstructor
@RestController
@RequestMapping("/als/falseAlarm")
public class FalseAlarmController extends BaseController {
    @Resource
    private IFalseAlarmService falseAlarmService;

    /**
     * 查询虚警抑制列表
     */
    @SaCheckPermission("als:falseAlarm:list")
    @GetMapping("/list")
    public CommonResult<PageResult<FalseAlarmVo>> list(FalseAlarmBo falseAlarmBo) {
        return CommonResult.success(falseAlarmService.selectPage(falseAlarmBo));
    }

    /**
     * 导出虚警抑制列表
     */
    @SaCheckPermission("als:falseAlarm:export")
    @Log(title = "虚警抑制", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public CommonResult<Void> export(FalseAlarmBo falseAlarmBo) {
        LoginUser loginUser = LoginHelper.getLoginUser();
        List<FalseAlarmVo> list = falseAlarmService.selectList(falseAlarmBo);
        falseAlarmService.asyncExport(list, "虚警抑制", loginUser);
        return CommonResult.success();
    }

    /**
     * 获取虚警抑制详细信息
     */
    @SaCheckPermission("als:falseAlarm:query")
    @GetMapping(value = "/{id}")
    public CommonResult<FalseAlarmVo> getInfo(@PathVariable Long id) {
        return CommonResult.success(falseAlarmService.selectById(id));
    }

    /**
     * 新增虚警抑制
     */
    @SaCheckPermission("als:falseAlarm:add")
    @Log(title = "虚警抑制", businessType = BusinessType.INSERT)
    @RepeatSubmit()
    @PostMapping
    public CommonResult<Void> add(@Validated @RequestBody FalseAlarmBo falseAlarmBo) {
        FalseAlarm falseAlarm = falseAlarmService.insert(falseAlarmBo);
        if (ObjectUtil.isEmpty(falseAlarm)) {
            return CommonResult.fail("新增虚警抑制记录失败！");
        }
        return CommonResult.success();
    }

    /**
     * 修改虚警抑制
     */
    @SaCheckPermission("als:falseAlarm:edit")
    @Log(title = "虚警抑制", businessType = BusinessType.UPDATE)
    @RepeatSubmit()
    @PutMapping
    public CommonResult<Void> edit(@Validated @RequestBody FalseAlarmBo falseAlarmBo) {
        boolean updated = falseAlarmService.update(falseAlarmBo);
        if (!updated) {
            return CommonResult.fail("修改虚警抑制记录失败!");
        }
        return CommonResult.success();
    }

    /**
     * 删除虚警抑制
     */
    @SaCheckPermission("als:falseAlarm:remove")
    @Log(title = "虚警抑制", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public CommonResult<Void> remove(@PathVariable Long[] ids) {
        boolean deleted = falseAlarmService.deleteByIds(ids);
        if (!deleted) {
            return CommonResult.fail("删除虚警抑制记录失败!");
        }
        return CommonResult.success();
    }
}
