package org.eco.als.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.ObjectUtil;
import jakarta.annotation.Resource;
import lombok.RequiredArgsConstructor;
import org.eco.als.domain.FaultDiagnosis;
import org.eco.als.domain.bo.FaultDiagnosisBo;
import org.eco.als.domain.vo.FaultDiagnosisVo;
import org.eco.als.service.IFaultDiagnosisService;
import org.eco.common.core.core.domain.CommonResult;
import org.eco.common.core.core.domain.model.LoginUser;
import org.eco.common.core.core.page.PageResult;
import org.eco.common.log.annotation.Log;
import org.eco.common.log.enums.BusinessType;
import org.eco.common.security.utils.LoginHelper;
import org.eco.common.web.annotation.RepeatSubmit;
import org.eco.common.web.core.BaseController;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * 故障诊断Controller
 *
 * @author wgk
 * @date 2024-07-23
 */
@Validated
@RequiredArgsConstructor
@RestController
@RequestMapping("/als/faultDiagnosis")
public class FaultDiagnosisController extends BaseController {
    @Resource
    private IFaultDiagnosisService faultDiagnosisService;

    /**
     * 查询故障诊断列表
     */
    @SaCheckPermission("als:faultDiagnosis:list")
    @GetMapping("/list")
    public CommonResult<PageResult<FaultDiagnosisVo>> list(FaultDiagnosisBo faultDiagnosisBo) {
        return CommonResult.success(faultDiagnosisService.selectPage(faultDiagnosisBo));
    }

    /**
     * 导出故障诊断列表
     */
    @SaCheckPermission("als:faultDiagnosis:export")
    @Log(title = "故障诊断", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public CommonResult<Void> export(FaultDiagnosisBo faultDiagnosisBo) {
        LoginUser loginUser = LoginHelper.getLoginUser();
        List<FaultDiagnosisVo> list = faultDiagnosisService.selectList(faultDiagnosisBo);
        if (CollUtil.isEmpty(list)) {
            return CommonResult.fail("导出列表为空");
        }
        faultDiagnosisService.asyncExport(list, "故障诊断", loginUser);
        return CommonResult.success();
    }

    /**
     * 获取故障诊断详细信息
     */
    @SaCheckPermission("als:faultDiagnosis:query")
    @GetMapping(value = "/{id}")
    public CommonResult<FaultDiagnosisVo> getInfo(@PathVariable Long id) {
        return CommonResult.success(faultDiagnosisService.selectById(id));
    }

    /**
     * 新增故障诊断
     */
    @SaCheckPermission("als:faultDiagnosis:add")
    @Log(title = "故障诊断", businessType = BusinessType.INSERT)
    @RepeatSubmit()
    @PostMapping
    public CommonResult<Void> add(@Validated @RequestBody FaultDiagnosisBo faultDiagnosisBo) {
        FaultDiagnosis faultDiagnosis = faultDiagnosisService.insert(faultDiagnosisBo);
        if (ObjectUtil.isEmpty(faultDiagnosis)) {
            return CommonResult.fail("新增故障诊断记录失败！");
        }
        return CommonResult.success();
    }

    /**
     * 修改故障诊断
     */
    @SaCheckPermission("als:faultDiagnosis:edit")
    @Log(title = "故障诊断", businessType = BusinessType.UPDATE)
    @RepeatSubmit()
    @PutMapping
    public CommonResult<Void> edit(@Validated @RequestBody FaultDiagnosisBo faultDiagnosisBo) {
        boolean updated = faultDiagnosisService.update(faultDiagnosisBo);
        if (!updated) {
            return CommonResult.fail("修改故障诊断记录失败!");
        }
        return CommonResult.success();
    }

    /**
     * 删除故障诊断
     */
    @SaCheckPermission("als:faultDiagnosis:remove")
    @Log(title = "故障诊断", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public CommonResult<Void> remove(@PathVariable Long[] ids) {
        boolean deleted = faultDiagnosisService.deleteByIds(ids);
        if (!deleted) {
            return CommonResult.fail("删除故障诊断记录失败!");
        }
        return CommonResult.success();
    }
}
