package org.eco.als.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import cn.hutool.core.collection.CollUtil;
import jakarta.annotation.Resource;
import lombok.RequiredArgsConstructor;
import org.eco.als.domain.bo.MaintenanceDecisionBo;
import org.eco.als.domain.vo.MaintenanceDecisionVo;
import org.eco.als.service.IMaintenanceDecisionService;
import org.eco.common.core.core.domain.CommonResult;
import org.eco.common.core.core.domain.model.LoginUser;
import org.eco.common.core.core.page.PageResult;
import org.eco.common.log.annotation.Log;
import org.eco.common.log.enums.BusinessType;
import org.eco.common.security.utils.LoginHelper;
import org.eco.common.web.annotation.RepeatSubmit;
import org.eco.common.web.core.BaseController;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * 辅助维修决策Controller
 *
 * @author wgk
 * @date 2024-07-24
 */
@Validated
@RequiredArgsConstructor
@RestController
@RequestMapping("/als/maintenanceDecision")
public class MaintenanceDecisionController extends BaseController {
    @Resource
    private IMaintenanceDecisionService maintenanceDecisionService;

    /**
     * 查询辅助维修决策列表
     */
    @SaCheckPermission("als:maintenanceDecision:list")
    @GetMapping("/list")
    public CommonResult<PageResult<MaintenanceDecisionVo>> list(MaintenanceDecisionBo maintenanceDecisionBo) {
        return CommonResult.success(maintenanceDecisionService.selectPage(maintenanceDecisionBo));
    }

    /**
     * 导出辅助维修决策列表
     */
    @SaCheckPermission("als:maintenanceDecision:export")
    @Log(title = "辅助维修决策", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public CommonResult<Void> export(MaintenanceDecisionBo maintenanceDecisionBo) {
        LoginUser loginUser = LoginHelper.getLoginUser();
        List<MaintenanceDecisionVo> list = maintenanceDecisionService.selectList(maintenanceDecisionBo);
        if (CollUtil.isEmpty(list)) {
            return CommonResult.fail("导出列表为空");
        }
        maintenanceDecisionService.asyncExport(list, "辅助维修决策", loginUser);
        return CommonResult.success();
    }

    /**
     * 获取辅助维修决策详细信息
     */
    @SaCheckPermission("als:maintenanceDecision:query")
    @GetMapping(value = "/{id}")
    public CommonResult<MaintenanceDecisionVo> getInfo(@PathVariable Long id) {
        return CommonResult.success(maintenanceDecisionService.selectById(id));
    }

    /**
     * 新增辅助维修决策
     */
    @SaCheckPermission("als:maintenanceDecision:add")
    @Log(title = "辅助维修决策", businessType = BusinessType.INSERT)
    @RepeatSubmit()
    @PostMapping
    public CommonResult<Void> add(@Validated @RequestBody MaintenanceDecisionBo maintenanceDecisionBo) {
        boolean inserted = maintenanceDecisionService.insert(maintenanceDecisionBo);
        if (!inserted) {
            return CommonResult.fail("新增辅助维修决策记录失败！");
        }
        return CommonResult.success();
    }

    /**
     * 修改辅助维修决策
     */
    @SaCheckPermission("als:maintenanceDecision:edit")
    @Log(title = "辅助维修决策", businessType = BusinessType.UPDATE)
    @RepeatSubmit()
    @PutMapping
    public CommonResult<Void> edit(@Validated @RequestBody MaintenanceDecisionBo maintenanceDecisionBo) {
        boolean updated = maintenanceDecisionService.update(maintenanceDecisionBo);
        if (!updated) {
            return CommonResult.fail("修改辅助维修决策记录失败!");
        }
        return CommonResult.success();
    }

    /**
     * 删除辅助维修决策
     */
    @SaCheckPermission("als:maintenanceDecision:remove")
    @Log(title = "辅助维修决策", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public CommonResult<Void> remove(@PathVariable Long[] ids) {
        boolean deleted = maintenanceDecisionService.deleteByIds(ids);
        if (!deleted) {
            return CommonResult.fail("删除辅助维修决策记录失败!");
        }
        return CommonResult.success();
    }
}
