package org.eco.als.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import cn.hutool.core.collection.CollUtil;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletResponse;
import lombok.RequiredArgsConstructor;
import org.eco.als.domain.bo.MaintenanceTaskBo;
import org.eco.als.domain.bo.SubTaskBo;
import org.eco.als.domain.vo.MaintenanceTaskImportVo;
import org.eco.als.domain.vo.MaintenanceTaskVo;
import org.eco.als.service.IMaintenanceTaskService;
import org.eco.common.core.core.domain.CommonResult;
import org.eco.common.core.core.domain.model.LoginUser;
import org.eco.common.core.core.page.PageResult;
import org.eco.common.excel.utils.ExcelUtil;
import org.eco.common.log.annotation.Log;
import org.eco.common.log.enums.BusinessType;
import org.eco.common.security.utils.LoginHelper;
import org.eco.common.web.annotation.RepeatSubmit;
import org.eco.common.web.core.BaseController;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.util.ArrayList;
import java.util.List;

/**
 * 维修信息Controller
 *
 * @author wgk
 * @date 2024-11-14
 */
@Validated
@RequiredArgsConstructor
@RestController
@RequestMapping("/als/maintenanceask")
public class MaintenanceTaskController extends BaseController {
    @Resource
    private IMaintenanceTaskService maintenanceTaskService;

    /**
     * 查询维修信息列表
     */
    @SaCheckPermission("als:maintenanceask:list")
    @GetMapping("/list")
    public CommonResult<PageResult<MaintenanceTaskVo>> list(MaintenanceTaskBo maintenanceTaskBo) {
        return CommonResult.success(maintenanceTaskService.selectPage(maintenanceTaskBo));
    }

    /**
     * 导出维修信息列表
     */
    @SaCheckPermission("als:maintenanceask:export")
    @Log(title = "维修信息", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public CommonResult<Void> export(MaintenanceTaskBo maintenanceTaskBo) {
        LoginUser loginUser = LoginHelper.getLoginUser();
        List<MaintenanceTaskVo> list = maintenanceTaskService.selectList(maintenanceTaskBo);
        if (CollUtil.isEmpty(list)) {
            return CommonResult.fail("导出列表为空");
        }
        maintenanceTaskService.asyncExport(list, "维修信息", loginUser);
        return CommonResult.success();
    }

    @SaCheckPermission("als:maintenanceask:import")
    @PostMapping("/importTemplate")
    public void importTemplate(HttpServletResponse response) {
        ExcelUtil.exportExcel(new ArrayList<>(), "维修信息", MaintenanceTaskImportVo.class, response);
    }

    /**
     * 导入维修信息列表
     */
    @Log(title = "维修信息", businessType = BusinessType.IMPORT)
    @SaCheckPermission("als:maintenanceask:import")
    @PostMapping("/importData")
    public CommonResult<Void> importData(MultipartFile file, boolean updateSupport) {
        LoginUser loginUser = LoginHelper.getLoginUser();
        maintenanceTaskService.asyncImportData(file, updateSupport, loginUser);
        return CommonResult.success();
    }

    /**
     * 获取维修信息详细信息
     */
    @SaCheckPermission("als:maintenanceask:query")
    @GetMapping(value = "/{id}")
    public CommonResult<MaintenanceTaskVo> getInfo(@PathVariable Long id) {
        return CommonResult.success(maintenanceTaskService.selectById(id));
    }

    /**
     * 新增维修信息
     */
    @SaCheckPermission("als:maintenanceask:add")
    @Log(title = "维修信息", businessType = BusinessType.INSERT)
    @RepeatSubmit()
    @PostMapping
    public CommonResult<Void> add(@Validated @RequestBody MaintenanceTaskBo maintenanceTaskBo) {
        boolean inserted = maintenanceTaskService.insert(maintenanceTaskBo);
        if (!inserted) {
            return CommonResult.fail("新增维修信息记录失败！");
        }
        return CommonResult.success();
    }

    /**
     * 修改维修信息
     */
    @SaCheckPermission("als:maintenanceask:edit")
    @Log(title = "维修信息", businessType = BusinessType.UPDATE)
    @RepeatSubmit()
    @PutMapping
    public CommonResult<Void> edit(@Validated @RequestBody MaintenanceTaskBo maintenanceTaskBo) {
        boolean updated = maintenanceTaskService.update(maintenanceTaskBo);
        if (!updated) {
            return CommonResult.fail("修改维修信息记录失败!");
        }
        return CommonResult.success();
    }

    /**
     * 删除维修信息
     */
    @SaCheckPermission("als:maintenanceask:remove")
    @Log(title = "维修信息", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public CommonResult<Void> remove(@PathVariable Long[] ids) {
        boolean deleted = maintenanceTaskService.deleteByIds(ids);
        if (!deleted) {
            return CommonResult.fail("删除维修信息记录失败!");
        }
        return CommonResult.success();
    }

    /**
     * 任务派遣 资源分配
     *
     * @param subTaskBo 子任务
     * @return res
     */
    @PostMapping("/allocation")
    public CommonResult<Void> allocation(SubTaskBo subTaskBo) {
        boolean updated = maintenanceTaskService.allocation(subTaskBo);
        return updated ? CommonResult.success() : CommonResult.fail();
    }
}
