package org.eco.als.controller;

import lombok.RequiredArgsConstructor;
import jakarta.servlet.http.HttpServletResponse;
import cn.hutool.core.collection.CollUtil;
import cn.dev33.satoken.annotation.SaCheckPermission;
import org.springframework.web.bind.annotation.*;
import org.springframework.validation.annotation.Validated;
import org.eco.common.core.core.domain.CommonResult;
import org.eco.common.log.annotation.Log;
import org.eco.common.log.enums.BusinessType;
import org.eco.common.security.utils.LoginHelper;
import org.eco.common.core.core.domain.model.LoginUser;
import org.eco.common.web.annotation.RepeatSubmit;
import org.eco.common.excel.utils.ExcelUtil;
import org.eco.common.web.core.BaseController;
import jakarta.annotation.Resource;
import org.eco.als.domain.vo.PartReplacementImportVo;
import org.eco.als.domain.vo.PartReplacementVo;
import org.eco.als.domain.bo.PartReplacementBo;
import org.eco.als.service.IPartReplacementService;
import org.springframework.web.multipart.MultipartFile;
import org.eco.common.core.core.page.PageResult;

import java.util.List;
import java.util.ArrayList;

/**
 * 换件记录Controller
 *
 * @author wgk
 * @date 2024-10-25
 */
@Validated
@RequiredArgsConstructor
@RestController
@RequestMapping("/als/partReplacement")
public class PartReplacementController extends BaseController {
    @Resource
    private IPartReplacementService partReplacementService;

    /**
     * 查询换件记录列表
     */
    @SaCheckPermission("als:partReplacement:list")
    @GetMapping("/list")
    public CommonResult<PageResult<PartReplacementVo>> list(PartReplacementBo partReplacementBo) {
        return CommonResult.success(partReplacementService.selectPage(partReplacementBo));
    }

    /**
     * 导出换件记录列表
     */
    @SaCheckPermission("als:partReplacement:export")
    @Log(title = "换件记录", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public CommonResult<Void> export(PartReplacementBo partReplacementBo) {
        LoginUser loginUser = LoginHelper.getLoginUser();
        List<PartReplacementVo> list = partReplacementService.selectList(partReplacementBo);
        if (CollUtil.isEmpty(list)) {
            return CommonResult.fail("导出列表为空");
        }
        partReplacementService.asyncExport(list, "换件记录", loginUser);
        return CommonResult.success();
    }

    @SaCheckPermission("als:partReplacement:import")
    @PostMapping("/importTemplate")
    public void importTemplate(HttpServletResponse response) {
        ExcelUtil.exportExcel(new ArrayList<>(), "换件记录", PartReplacementImportVo.class, response);
    }

    /**
     * 导入换件记录列表
     */
    @Log(title = "换件记录", businessType = BusinessType.IMPORT)
    @SaCheckPermission("als:partReplacement:import")
    @PostMapping("/importData")
    public CommonResult<Void> importData(MultipartFile file, boolean updateSupport) {
        LoginUser loginUser = LoginHelper.getLoginUser();
        partReplacementService.asyncImportData(file, updateSupport, loginUser);
        return CommonResult.success();
    }

    /**
     * 获取换件记录详细信息
     */
    @SaCheckPermission("als:partReplacement:query")
    @GetMapping(value = "/{id}")
    public CommonResult<PartReplacementVo> getInfo(@PathVariable Long id) {
        return CommonResult.success(partReplacementService.selectById(id));
    }

    /**
     * 新增换件记录
     */
    @SaCheckPermission("als:partReplacement:add")
    @Log(title = "换件记录", businessType = BusinessType.INSERT)
    @RepeatSubmit()
    @PostMapping
    public CommonResult<Void> add(@Validated @RequestBody PartReplacementBo partReplacementBo) {
        boolean inserted = partReplacementService.insert(partReplacementBo);
        if (!inserted) {
            return CommonResult.fail("新增换件记录记录失败！");
        }
        return CommonResult.success();
    }

    /**
     * 修改换件记录
     */
    @SaCheckPermission("als:partReplacement:edit")
    @Log(title = "换件记录", businessType = BusinessType.UPDATE)
    @RepeatSubmit()
    @PutMapping
    public CommonResult<Void> edit(@Validated @RequestBody PartReplacementBo partReplacementBo) {
        boolean updated = partReplacementService.update(partReplacementBo);
        if (!updated) {
            return CommonResult.fail("修改换件记录记录失败!");
        }
        return CommonResult.success();
    }

    /**
     * 删除换件记录
     */
    @SaCheckPermission("als:partReplacement:remove")
    @Log(title = "换件记录", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public CommonResult<Void> remove(@PathVariable Long[] ids) {
        boolean deleted = partReplacementService.deleteByIds(ids);
        if (!deleted) {
            return CommonResult.fail("删除换件记录记录失败!");
        }
        return CommonResult.success();
    }
}
