package org.eco.als.controller;

import lombok.RequiredArgsConstructor;
import jakarta.servlet.http.HttpServletResponse;
import cn.hutool.core.collection.CollUtil;
import cn.dev33.satoken.annotation.SaCheckPermission;
import org.springframework.web.bind.annotation.*;
import org.springframework.validation.annotation.Validated;
import org.eco.common.core.core.domain.CommonResult;
import org.eco.common.log.annotation.Log;
import org.eco.common.log.enums.BusinessType;
import org.eco.common.security.utils.LoginHelper;
import org.eco.common.core.core.domain.model.LoginUser;
import org.eco.common.web.annotation.RepeatSubmit;
import org.eco.common.excel.utils.ExcelUtil;
import org.eco.common.web.core.BaseController;
import jakarta.annotation.Resource;
import org.eco.als.domain.vo.PersonnelAllocationImportVo;
import org.eco.als.domain.vo.PersonnelAllocationVo;
import org.eco.als.domain.bo.PersonnelAllocationBo;
import org.eco.als.service.IPersonnelAllocationService;
import org.springframework.web.multipart.MultipartFile;
import org.eco.common.core.core.page.PageResult;

import java.util.List;
import java.util.ArrayList;

/**
 * 维修任务任务派遣Controller
 *
 * @author wgk
 * @date 2024-11-14
 */
@Validated
@RequiredArgsConstructor
@RestController
@RequestMapping("/als/personnelAllocation")
public class PersonnelAllocationController extends BaseController {
    @Resource
    private IPersonnelAllocationService personnelAllocationService;

    /**
     * 查询维修任务任务派遣列表
     */
    @SaCheckPermission("als:personnelAllocation:list")
    @GetMapping("/list")
    public CommonResult<PageResult<PersonnelAllocationVo>> list(PersonnelAllocationBo personnelAllocationBo) {
        return CommonResult.success(personnelAllocationService.selectPage(personnelAllocationBo));
    }

    /**
     * 导出维修任务任务派遣列表
     */
    @SaCheckPermission("als:personnelAllocation:export")
    @Log(title = "维修任务任务派遣", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public CommonResult<Void> export(PersonnelAllocationBo personnelAllocationBo) {
        LoginUser loginUser = LoginHelper.getLoginUser();
        List<PersonnelAllocationVo> list = personnelAllocationService.selectList(personnelAllocationBo);
        if (CollUtil.isEmpty(list)) {
            return CommonResult.fail("导出列表为空");
        }
        personnelAllocationService.asyncExport(list, "维修任务任务派遣", loginUser);
        return CommonResult.success();
    }

    @SaCheckPermission("als:personnelAllocation:import")
    @PostMapping("/importTemplate")
    public void importTemplate(HttpServletResponse response) {
        ExcelUtil.exportExcel(new ArrayList<>(), "维修任务任务派遣", PersonnelAllocationImportVo.class, response);
    }

    /**
     * 导入维修任务任务派遣列表
     */
    @Log(title = "维修任务任务派遣", businessType = BusinessType.IMPORT)
    @SaCheckPermission("als:personnelAllocation:import")
    @PostMapping("/importData")
    public CommonResult<Void> importData(MultipartFile file, boolean updateSupport) {
        LoginUser loginUser = LoginHelper.getLoginUser();
        personnelAllocationService.asyncImportData(file, updateSupport, loginUser);
        return CommonResult.success();
    }

    /**
     * 获取维修任务任务派遣详细信息
     */
    @SaCheckPermission("als:personnelAllocation:query")
    @GetMapping(value = "/{id}")
    public CommonResult<PersonnelAllocationVo> getInfo(@PathVariable Long id) {
        return CommonResult.success(personnelAllocationService.selectById(id));
    }

    /**
     * 新增维修任务任务派遣
     */
    @SaCheckPermission("als:personnelAllocation:add")
    @Log(title = "维修任务任务派遣", businessType = BusinessType.INSERT)
    @RepeatSubmit()
    @PostMapping
    public CommonResult<Void> add(@Validated @RequestBody PersonnelAllocationBo personnelAllocationBo) {
        boolean inserted = personnelAllocationService.insert(personnelAllocationBo);
        if (!inserted) {
            return CommonResult.fail("新增维修任务任务派遣记录失败！");
        }
        return CommonResult.success();
    }

    /**
     * 修改维修任务任务派遣
     */
    @SaCheckPermission("als:personnelAllocation:edit")
    @Log(title = "维修任务任务派遣", businessType = BusinessType.UPDATE)
    @RepeatSubmit()
    @PutMapping
    public CommonResult<Void> edit(@Validated @RequestBody PersonnelAllocationBo personnelAllocationBo) {
        boolean updated = personnelAllocationService.update(personnelAllocationBo);
        if (!updated) {
            return CommonResult.fail("修改维修任务任务派遣记录失败!");
        }
        return CommonResult.success();
    }

    /**
     * 删除维修任务任务派遣
     */
    @SaCheckPermission("als:personnelAllocation:remove")
    @Log(title = "维修任务任务派遣", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public CommonResult<Void> remove(@PathVariable Long[] ids) {
        boolean deleted = personnelAllocationService.deleteByIds(ids);
        if (!deleted) {
            return CommonResult.fail("删除维修任务任务派遣记录失败!");
        }
        return CommonResult.success();
    }
}
