package org.eco.als.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletResponse;
import lombok.RequiredArgsConstructor;
import org.eco.als.domain.bo.ProductBo;
import org.eco.als.domain.vo.ProductImportVo;
import org.eco.als.domain.vo.ProductVo;
import org.eco.als.service.IProductService;
import org.eco.common.core.core.domain.CommonResult;
import org.eco.common.core.core.domain.model.LoginUser;
import org.eco.common.excel.utils.ExcelUtil;
import org.eco.common.log.annotation.Log;
import org.eco.common.log.enums.BusinessType;
import org.eco.common.security.utils.LoginHelper;
import org.eco.common.web.annotation.RepeatSubmit;
import org.eco.common.web.core.BaseController;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import java.util.ArrayList;
import java.util.List;

/**
 * 产品树Controller
 *
 * @author wgk
 * @date 2024-07-19
 */
@Validated
@RequiredArgsConstructor
@RestController
@RequestMapping("/als/product")
public class ProductController extends BaseController {
    @Resource
    private IProductService productService;

    /**
     * 查询产品树列表
     */
    @SaCheckPermission("als:product:list")
    @GetMapping("/list")
    public CommonResult<List<ProductVo>> list(ProductBo productBo) {
        List<ProductVo> list = productService.selectList(productBo);
        return CommonResult.success(list);
    }

    /**
     * 导出产品树列表
     */
    @SaCheckPermission("als:product:export")
    @Log(title = "产品树", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public CommonResult<Void> export(ProductBo productBo) {
        LoginUser loginUser = LoginHelper.getLoginUser();
        List<ProductVo> list = productService.selectList(productBo);
        productService.asyncExport(list, "产品树", loginUser);
        return CommonResult.success();
    }

    @SaCheckPermission("als:product:import")
    @PostMapping("/importTemplate")
    public void importTemplate(HttpServletResponse response) {
        ExcelUtil.exportExcel(new ArrayList<>(), "产品树", ProductImportVo.class, response);
    }

    /**
     * 导入产品树列表
     */
    @Log(title = "产品树", businessType = BusinessType.IMPORT)
    @SaCheckPermission("als:product:import")
    @PostMapping("/importData")
    public CommonResult<Void> importData(MultipartFile file, boolean updateSupport) {
        LoginUser loginUser = LoginHelper.getLoginUser();
        productService.asyncImportData(file, updateSupport, loginUser);
        return CommonResult.success();
    }

    /**
     * 获取产品树详细信息
     */
    @SaCheckPermission("als:product:query")
    @GetMapping(value = "/{id}")
    public CommonResult<ProductVo> getInfo(@PathVariable Long id) {
        return CommonResult.success(productService.selectById(id));
    }

    /**
     * 新增产品树
     */
    @SaCheckPermission("als:product:add")
    @Log(title = "产品树", businessType = BusinessType.INSERT)
    @RepeatSubmit()
    @PostMapping
    public CommonResult<Void> add(@Validated @RequestBody ProductBo productBo) {
        boolean inserted = productService.insert(productBo);
        if (!inserted) {
            return CommonResult.fail("新增产品树记录失败！");
        }
        return CommonResult.success();
    }

    /**
     * 修改产品树
     */
    @SaCheckPermission("als:product:edit")
    @Log(title = "产品树", businessType = BusinessType.UPDATE)
    @RepeatSubmit()
    @PutMapping
    public CommonResult<Void> edit(@Validated @RequestBody ProductBo productBo) {
        boolean updated = productService.update(productBo);
        if (!updated) {
            return CommonResult.fail("修改产品树记录失败!");
        }
        return CommonResult.success();
    }

    /**
     * 删除产品树
     */
    @SaCheckPermission("als:product:remove")
    @Log(title = "产品树", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public CommonResult<Void> remove(@PathVariable Long[] ids) {
        boolean deleted = productService.deleteByIds(ids);
        if (!deleted) {
            return CommonResult.fail("删除产品树记录失败!");
        }
        return CommonResult.success();
    }
}
