package org.eco.als.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import cn.hutool.core.collection.CollUtil;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletResponse;
import lombok.RequiredArgsConstructor;
import org.eco.als.domain.bo.FaultStatisticsBo;
import org.eco.als.domain.bo.SmTaskBo;
import org.eco.als.domain.bo.SubTaskBo;
import org.eco.als.domain.vo.FaultStatisticsImportVo;
import org.eco.als.domain.vo.FaultStatisticsVo;
import org.eco.als.domain.vo.SmTaskImportVo;
import org.eco.als.domain.vo.SmTaskVo;
import org.eco.als.listener.FaultStatisticsImportListener;
import org.eco.als.listener.SmTaskImportListener;
import org.eco.als.service.ISmTaskService;
import org.eco.common.core.core.domain.CommonResult;
import org.eco.common.core.core.domain.model.LoginUser;
import org.eco.common.core.core.page.PageResult;
import org.eco.common.excel.core.ExcelResult;
import org.eco.common.excel.utils.ExcelUtil;
import org.eco.common.log.annotation.Log;
import org.eco.common.log.enums.BusinessType;
import org.eco.common.security.utils.LoginHelper;
import org.eco.common.web.annotation.RepeatSubmit;
import org.eco.common.web.core.BaseController;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.util.ArrayList;
import java.util.List;

/**
 * 定检任务Controller
 *
 * @author wgk
 * @date 2024-11-20
 */
@Validated
@RequiredArgsConstructor
@RestController
@RequestMapping("/als/smTask")
public class SmTaskController extends BaseController {
    @Resource
    private ISmTaskService smTaskService;

    /**
     * 查询定检任务列表
     */
    @SaCheckPermission("als:smTask:list")
    @GetMapping("/list")
    public CommonResult<PageResult<SmTaskVo>> list(SmTaskBo smTaskBo) {
        return CommonResult.success(smTaskService.selectPage(smTaskBo));
    }

    /**
     * 导出定检任务列表
     */
    @SaCheckPermission("als:smTask:export")
    @Log(title = "定检任务", businessType = BusinessType.EXPORT)
    @GetMapping("/export")
    public void export(HttpServletResponse response, SmTaskBo smTaskBo) {
        List<SmTaskVo> list = smTaskService.selectList(smTaskBo);
        ExcelUtil.exportExcel(list, "定检任务", SmTaskVo.class, response);
    }

    @SaCheckPermission("als:smTask:import")
    @GetMapping("/importTemplate")
    public void importTemplate(HttpServletResponse response) {
        ExcelUtil.exportExcel(new ArrayList<>(), "定检任务", SmTaskImportVo.class, response);
    }

    /**
     * 导入定检任务列表
     */
    @Log(title = "定检任务", businessType = BusinessType.IMPORT)
    @SaCheckPermission("als:smTask:import")
    @PostMapping("/importData")
    public CommonResult<Void> importData(MultipartFile file, boolean updateSupport) throws Exception {
        LoginUser loginUser = LoginHelper.getLoginUser();
        ExcelResult<SmTaskImportVo> result = ExcelUtil.importExcel(file.getInputStream(), SmTaskImportVo.class, new SmTaskImportListener(updateSupport, loginUser));
        return CommonResult.success(result.getAnalysis().getLogInfo());
    }

    /**
     * 获取定检任务详细信息
     */
    @SaCheckPermission("als:smTask:query")
    @GetMapping(value = "/{id}")
    public CommonResult<SmTaskVo> getInfo(@PathVariable Long id) {
        return CommonResult.success(smTaskService.selectById(id));
    }

    /**
     * 新增定检任务
     */
    @SaCheckPermission("als:smTask:add")
    @Log(title = "定检任务", businessType = BusinessType.INSERT)
    @RepeatSubmit()
    @PostMapping
    public CommonResult<Void> add(@Validated @RequestBody SmTaskBo smTaskBo) {
        boolean inserted = smTaskService.insert(smTaskBo);
        if (!inserted) {
            return CommonResult.fail("新增定检任务记录失败！");
        }
        return CommonResult.success();
    }

    /**
     * 修改定检任务
     */
    @SaCheckPermission("als:smTask:edit")
    @Log(title = "定检任务", businessType = BusinessType.UPDATE)
    @RepeatSubmit()
    @PutMapping
    public CommonResult<Void> edit(@Validated @RequestBody SmTaskBo smTaskBo) {
        boolean updated = smTaskService.update(smTaskBo);
        if (!updated) {
            return CommonResult.fail("修改定检任务记录失败!");
        }
        return CommonResult.success();
    }

    /**
     * 删除定检任务
     */
    @SaCheckPermission("als:smTask:remove")
    @Log(title = "定检任务", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public CommonResult<Void> remove(@PathVariable Long[] ids) {
        boolean deleted = smTaskService.deleteByIds(ids);
        if (!deleted) {
            return CommonResult.fail("删除定检任务记录失败!");
        }
        return CommonResult.success();
    }

    /**
     * 任务派遣 资源分配
     *
     * @param subTaskBo 子任务
     * @return res
     */
    @PostMapping("/allocation")
    public CommonResult<Void> allocation(SubTaskBo subTaskBo) {
        boolean updated = smTaskService.allocation(subTaskBo);
        return updated ? CommonResult.success() : CommonResult.fail();
    }
}
