package org.eco.als.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import cn.hutool.core.collection.CollUtil;
import jakarta.annotation.Resource;
import lombok.RequiredArgsConstructor;
import org.eco.als.domain.bo.TaskBo;
import org.eco.als.domain.vo.TaskVo;
import org.eco.als.service.ITaskService;
import org.eco.common.core.core.domain.CommonResult;
import org.eco.common.core.core.domain.model.LoginUser;
import org.eco.common.core.core.page.PageResult;
import org.eco.common.log.annotation.Log;
import org.eco.common.log.enums.BusinessType;
import org.eco.common.security.utils.LoginHelper;
import org.eco.common.web.annotation.RepeatSubmit;
import org.eco.common.web.core.BaseController;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * 任务管理Controller
 *
 * @author wgk
 * @date 2024-08-07
 */
@Validated
@RequiredArgsConstructor
@RestController
@RequestMapping("/als/task")
public class TaskController extends BaseController {
    @Resource
    private ITaskService taskService;

    /**
     * 查询任务管理列表
     */
    @SaCheckPermission("als:task:list")
    @GetMapping("/list")
    public CommonResult<PageResult<TaskVo>> list(TaskBo taskBo) {
        return CommonResult.success(taskService.selectPage(taskBo));
    }

    /**
     * 导出任务管理列表
     */
    @SaCheckPermission("als:task:export")
    @Log(title = "任务管理", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public CommonResult<Void> export(TaskBo taskBo) {
        LoginUser loginUser = LoginHelper.getLoginUser();
        List<TaskVo> list = taskService.selectList(taskBo);
        if (CollUtil.isEmpty(list)) {
            return CommonResult.fail("导出列表为空");
        }
        taskService.asyncExport(list, "任务管理", loginUser);
        return CommonResult.success();
    }

    /**
     * 获取任务管理详细信息
     */
    @SaCheckPermission("als:task:query")
    @GetMapping(value = "/{id}")
    public CommonResult<TaskVo> getInfo(@PathVariable Long id) {
        return CommonResult.success(taskService.selectById(id));
    }

    /**
     * 新增任务管理
     */
    @SaCheckPermission("als:task:add")
    @Log(title = "任务管理", businessType = BusinessType.INSERT)
    @RepeatSubmit()
    @PostMapping
    public CommonResult<Void> add(@Validated @RequestBody TaskBo taskBo) {
        boolean inserted = taskService.insert(taskBo);
        if (!inserted) {
            return CommonResult.fail("新增任务管理记录失败！");
        }
        return CommonResult.success();
    }

    /**
     * 修改任务管理
     */
    @SaCheckPermission("als:task:edit")
    @Log(title = "任务管理", businessType = BusinessType.UPDATE)
    @RepeatSubmit()
    @PutMapping
    public CommonResult<Void> edit(@Validated @RequestBody TaskBo taskBo) {
        boolean updated = taskService.update(taskBo);
        if (!updated) {
            return CommonResult.fail("修改任务管理记录失败!");
        }
        return CommonResult.success();
    }

    /**
     * 删除任务管理
     */
    @SaCheckPermission("als:task:remove")
    @Log(title = "任务管理", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public CommonResult<Void> remove(@PathVariable Long[] ids) {
        boolean deleted = taskService.deleteByIds(ids);
        if (!deleted) {
            return CommonResult.fail("删除任务管理记录失败!");
        }
        return CommonResult.success();
    }
}
