package org.eco.als.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.mybatisflex.core.paginate.Page;
import com.mybatisflex.core.query.QueryWrapper;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.eco.als.domain.Contacts;
import org.eco.als.domain.bo.ContactsBo;
import org.eco.als.domain.vo.ContactsImportVo;
import org.eco.als.domain.vo.ContactsVo;
import org.eco.als.listener.ContactsImportListener;
import org.eco.als.mapper.ContactsMapper;
import org.eco.als.service.IContactsService;
import org.eco.common.core.core.domain.model.LoginUser;
import org.eco.common.core.core.page.PageResult;
import org.eco.common.core.utils.MapstructUtils;
import org.eco.common.excel.entity.ExcelResultRes;
import org.eco.common.excel.service.IExcelService;
import org.eco.common.orm.core.page.PageQuery;
import org.eco.common.orm.core.service.impl.BaseServiceImpl;
import org.eco.system.service.IImportExportService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.Arrays;
import java.util.List;

import static org.eco.als.domain.table.ContactsTableDef.CONTACTS;

/**
 * 通讯录Service业务层处理
 *
 * @author wgk
 * @date 2024-11-25
 */
@Service
@Slf4j
public class ContactsServiceImpl extends BaseServiceImpl<ContactsMapper, Contacts> implements IContactsService {
    @Resource
    private ContactsMapper contactsMapper;

    @Resource
    private IExcelService excelService;

    @Resource
    private IImportExportService importExportService;

    @Override
    public QueryWrapper query() {
        return super.query().from(CONTACTS);
    }

    private QueryWrapper buildQueryWrapper(ContactsBo contactsBo) {
        QueryWrapper queryWrapper = super.buildBaseQueryWrapper();
        queryWrapper.and(CONTACTS.NAME.like
            (contactsBo.getName()));
        queryWrapper.and(CONTACTS.URL.eq
            (contactsBo.getUrl()));
        queryWrapper.and(CONTACTS.PORT.eq
            (contactsBo.getPort()));
        queryWrapper.and(CONTACTS.PARAMETER.eq
            (contactsBo.getParameter()));
        queryWrapper.and(CONTACTS.REMARKS.eq
            (contactsBo.getRemarks()));

        return queryWrapper;
    }

    /**
     * 查询通讯录
     *
     * @param id 通讯录主键
     * @return 通讯录
     */
    @Override
    public ContactsVo selectById(Long id) {
        return this.getOneAs(query().where(CONTACTS.ID.eq(id)), ContactsVo.class);

    }


    /**
     * 查询通讯录列表
     *
     * @param contactsBo 通讯录Bo
     * @return 通讯录集合
     */
    @Override
    public List<ContactsVo> selectList(ContactsBo contactsBo) {
        QueryWrapper queryWrapper = buildQueryWrapper(contactsBo);
        return this.listAs(queryWrapper, ContactsVo.class);
    }

    /**
     * 分页查询通讯录列表
     *
     * @param contactsBo 通讯录Bo
     * @return 分页通讯录集合
     */
    @Override
    public PageResult<ContactsVo> selectPage(ContactsBo contactsBo) {
        QueryWrapper queryWrapper = buildQueryWrapper(contactsBo);
        Page<ContactsVo> page = this.pageAs(PageQuery.build(), queryWrapper, ContactsVo.class);
        return PageResult.build(page);
    }

    /**
     * 新增通讯录
     *
     * @param contactsBo 通讯录Bo
     * @return 结果:true 操作成功，false 操作失败
     */
    @Override
    public boolean insert(ContactsBo contactsBo) {
        Contacts contacts = MapstructUtils.convert(contactsBo, Contacts.class);

        return this.save(contacts);//使用全局配置的雪花算法主键生成器生成ID值
    }

    /**
     * 新增通讯录，前台提供主键值，一般用于导入的场合
     *
     * @param contactsBo 通讯录Bo
     * @return 结果:true 操作成功，false 操作失败
     */
    @Override
    public boolean insertWithPk(ContactsBo contactsBo) {
        Contacts contacts = MapstructUtils.convert(contactsBo, Contacts.class);


        return contactsMapper.insertWithPk(contacts) > 0;//前台传来主键值
    }

    /**
     * 修改通讯录
     *
     * @param contactsBo 通讯录Bo
     * @return 结果:true 更新成功，false 更新失败
     */
    @Override
    public boolean update(ContactsBo contactsBo) {
        Contacts contacts = MapstructUtils.convert(contactsBo, Contacts.class);
        if (ObjectUtil.isNotNull(contacts) && ObjectUtil.isNotNull(contacts.getId())) {
            boolean updated = this.updateById(contacts);
            return updated;
        }
        return false;
    }

    @Override
    public void asyncImportData(MultipartFile file, boolean updateSupport, LoginUser loginUser) {
        ExcelResultRes result;
        try {
            String name = file.getOriginalFilename();
            result = excelService.importExcel(file.getInputStream(), name, ContactsImportVo.class, new ContactsImportListener(updateSupport, loginUser));
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        boolean flag = importExportService.saveInfo(result, loginUser, "0");
        if (flag) {
            log.info("异步导入日志写入成功");
        }
    }

    @Override
    public void asyncExport(List<ContactsVo> listVo, String sheetName, LoginUser loginUser) {
        ExcelResultRes result = excelService.exportExcel(listVo, sheetName, ContactsVo.class);
        boolean flag = importExportService.saveInfo(result, loginUser, "1");
        if (flag) {
            log.info("异步导出日志写入成功");
        }
    }

    /**
     * 批量删除通讯录
     *
     * @param ids 需要删除的通讯录主键集合
     * @return 结果:true 删除成功，false 删除失败
     */
    @Transactional
    @Override
    public boolean deleteByIds(Long[] ids) {
        return this.removeByIds(Arrays.asList(ids));
    }

}
