package org.eco.als.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.mybatisflex.core.paginate.Page;
import com.mybatisflex.core.query.QueryWrapper;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.eco.als.domain.FaultDiagnosisResult;
import org.eco.als.domain.bo.FaultDiagnosisResultBo;
import org.eco.als.domain.vo.FaultDiagnosisResultVo;
import org.eco.als.mapper.FaultDiagnosisResultMapper;
import org.eco.als.service.IFaultDiagnosisResultService;
import org.eco.common.core.core.page.PageResult;
import org.eco.common.core.utils.MapstructUtils;
import org.eco.common.orm.core.page.PageQuery;
import org.eco.common.orm.core.service.impl.BaseServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Arrays;
import java.util.List;

import static org.eco.als.domain.table.FaultDiagnosisResultTableDef.FAULT_DIAGNOSIS_RESULT;

/**
 * 故障诊断结果Service业务层处理
 *
 * @author wgk
 * @date 2024-11-25
 */
@Service
@Slf4j
public class FaultDiagnosisResultServiceImpl extends BaseServiceImpl<FaultDiagnosisResultMapper, FaultDiagnosisResult> implements IFaultDiagnosisResultService {
    @Resource
    private FaultDiagnosisResultMapper faultDiagnosisResultMapper;

    @Override
    public QueryWrapper query() {
        return super.query().from(FAULT_DIAGNOSIS_RESULT);
    }

    private QueryWrapper buildQueryWrapper(FaultDiagnosisResultBo faultDiagnosisResultBo) {
        QueryWrapper queryWrapper = super.buildBaseQueryWrapper();
        queryWrapper.and(FAULT_DIAGNOSIS_RESULT.DIAGNOSIS_ID.eq
            (faultDiagnosisResultBo.getDiagnosisId()));
        queryWrapper.and(FAULT_DIAGNOSIS_RESULT.SORTIE_NO.eq
            (faultDiagnosisResultBo.getSortieNo()));
        queryWrapper.and(FAULT_DIAGNOSIS_RESULT.AIRCRAFT_ID.eq
            (faultDiagnosisResultBo.getAircraftId()));
        queryWrapper.and(FAULT_DIAGNOSIS_RESULT.MODEL_TYPE.eq
            (faultDiagnosisResultBo.getModelType()));
        queryWrapper.and(FAULT_DIAGNOSIS_RESULT.PARAM.eq
            (faultDiagnosisResultBo.getParam()));
        queryWrapper.and(FAULT_DIAGNOSIS_RESULT.COLUMN_DATA.eq
            (faultDiagnosisResultBo.getColumnData()));
        queryWrapper.and(FAULT_DIAGNOSIS_RESULT.OSS_ID.eq
            (faultDiagnosisResultBo.getOssId()));
        queryWrapper.and(FAULT_DIAGNOSIS_RESULT.RESULT_CONTENT.eq
            (faultDiagnosisResultBo.getResultContent()));
        queryWrapper.and(FAULT_DIAGNOSIS_RESULT.STATUS.eq
            (faultDiagnosisResultBo.getStatus()));
        queryWrapper.and(FAULT_DIAGNOSIS_RESULT.REMARKS.eq
            (faultDiagnosisResultBo.getRemarks()));

        return queryWrapper;
    }

    /**
     * 查询故障诊断结果
     *
     * @param id 故障诊断结果主键
     * @return 故障诊断结果
     */
    @Override
    public FaultDiagnosisResultVo selectById(Long id) {
        return this.getOneAs(query().where(FAULT_DIAGNOSIS_RESULT.ID.eq(id)), FaultDiagnosisResultVo.class);

    }


    /**
     * 查询故障诊断结果列表
     *
     * @param faultDiagnosisResultBo 故障诊断结果Bo
     * @return 故障诊断结果集合
     */
    @Override
    public List<FaultDiagnosisResultVo> selectList(FaultDiagnosisResultBo faultDiagnosisResultBo) {
        QueryWrapper queryWrapper = buildQueryWrapper(faultDiagnosisResultBo);
        return this.listAs(queryWrapper, FaultDiagnosisResultVo.class);
    }

    /**
     * 分页查询故障诊断结果列表
     *
     * @param faultDiagnosisResultBo 故障诊断结果Bo
     * @return 分页故障诊断结果集合
     */
    @Override
    public PageResult<FaultDiagnosisResultVo> selectPage(FaultDiagnosisResultBo faultDiagnosisResultBo) {
        QueryWrapper queryWrapper = buildQueryWrapper(faultDiagnosisResultBo);
        Page<FaultDiagnosisResultVo> page = this.pageAs(PageQuery.build(), queryWrapper, FaultDiagnosisResultVo.class);
        return PageResult.build(page);
    }

    /**
     * 新增故障诊断结果
     *
     * @param faultDiagnosisResultBo 故障诊断结果Bo
     * @return 结果:true 操作成功，false 操作失败
     */
    @Override
    public boolean insert(FaultDiagnosisResultBo faultDiagnosisResultBo) {
        FaultDiagnosisResult faultDiagnosisResult = MapstructUtils.convert(faultDiagnosisResultBo, FaultDiagnosisResult.class);

        return this.save(faultDiagnosisResult);//使用全局配置的雪花算法主键生成器生成ID值
    }

    /**
     * 新增故障诊断结果，前台提供主键值，一般用于导入的场合
     *
     * @param faultDiagnosisResultBo 故障诊断结果Bo
     * @return 结果:true 操作成功，false 操作失败
     */
    @Override
    public boolean insertWithPk(FaultDiagnosisResultBo faultDiagnosisResultBo) {
        FaultDiagnosisResult faultDiagnosisResult = MapstructUtils.convert(faultDiagnosisResultBo, FaultDiagnosisResult.class);


        return faultDiagnosisResultMapper.insertWithPk(faultDiagnosisResult) > 0;//前台传来主键值
    }

    /**
     * 修改故障诊断结果
     *
     * @param faultDiagnosisResultBo 故障诊断结果Bo
     * @return 结果:true 更新成功，false 更新失败
     */
    @Override
    public boolean update(FaultDiagnosisResultBo faultDiagnosisResultBo) {
        FaultDiagnosisResult faultDiagnosisResult = MapstructUtils.convert(faultDiagnosisResultBo, FaultDiagnosisResult.class);
        if (ObjectUtil.isNotNull(faultDiagnosisResult) && ObjectUtil.isNotNull(faultDiagnosisResult.getId())) {
            boolean updated = this.updateById(faultDiagnosisResult);
            return updated;
        }
        return false;
    }

    /**
     * 批量删除故障诊断结果
     *
     * @param ids 需要删除的故障诊断结果主键集合
     * @return 结果:true 删除成功，false 删除失败
     */
    @Transactional
    @Override
    public boolean deleteByIds(Long[] ids) {
        return this.removeByIds(Arrays.asList(ids));
    }

}
