package org.eco.als.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.mybatisflex.core.paginate.Page;
import com.mybatisflex.core.query.QueryWrapper;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.eco.als.domain.FaultDiagnosis;
import org.eco.als.domain.bo.FaultDiagnosisBo;
import org.eco.als.domain.vo.FaultDiagnosisVo;
import org.eco.als.mapper.FaultDiagnosisMapper;
import org.eco.als.service.IFaultDiagnosisService;
import org.eco.common.core.core.domain.model.LoginUser;
import org.eco.common.core.core.page.PageResult;
import org.eco.common.core.utils.MapstructUtils;
import org.eco.common.core.utils.StringUtils;
import org.eco.common.excel.entity.ExcelResultRes;
import org.eco.common.excel.service.IExcelService;
import org.eco.common.orm.core.page.PageQuery;
import org.eco.common.orm.core.service.impl.BaseServiceImpl;
import org.eco.system.service.IImportExportService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Arrays;
import java.util.List;

import static org.eco.als.domain.table.FaultDiagnosisTableDef.FAULT_DIAGNOSIS;

/**
 * 故障诊断Service业务层处理
 *
 * @author wgk
 * @date 2024-07-23
 */
@Service
@Slf4j
public class FaultDiagnosisServiceImpl extends BaseServiceImpl<FaultDiagnosisMapper, FaultDiagnosis> implements IFaultDiagnosisService {
    @Resource
    private FaultDiagnosisMapper faultDiagnosisMapper;

    @Resource
    private IExcelService excelService;

    @Resource
    private IImportExportService importExportService;

    @Override
    public QueryWrapper query() {
        return super.query().from(FAULT_DIAGNOSIS);
    }

    private QueryWrapper buildQueryWrapper(FaultDiagnosisBo faultDiagnosisBo) {
        QueryWrapper queryWrapper = super.buildBaseQueryWrapper();
        queryWrapper.and(FAULT_DIAGNOSIS.SORTIE_NO.eq
            (faultDiagnosisBo.getSortieNo()));
        if (StringUtils.isNotBlank(faultDiagnosisBo.getAircraftId())) {
            queryWrapper.and(FAULT_DIAGNOSIS.AIRCRAFT_ID.in
                (faultDiagnosisBo.getAircraftId().split(",")));
        }

        return queryWrapper;
    }

    /**
     * 查询故障诊断
     *
     * @param id 故障诊断主键
     * @return 故障诊断
     */
    @Override
    public FaultDiagnosisVo selectById(Long id) {
        return this.getOneAs(query().where(FAULT_DIAGNOSIS.ID.eq(id)), FaultDiagnosisVo.class);

    }


    /**
     * 查询故障诊断列表
     *
     * @param faultDiagnosisBo 故障诊断Bo
     * @return 故障诊断集合
     */
    @Override
    public List<FaultDiagnosisVo> selectList(FaultDiagnosisBo faultDiagnosisBo) {
        QueryWrapper queryWrapper = buildQueryWrapper(faultDiagnosisBo);
        return this.listAs(queryWrapper, FaultDiagnosisVo.class);
    }

    /**
     * 分页查询故障诊断列表
     *
     * @param faultDiagnosisBo 故障诊断Bo
     * @return 分页故障诊断集合
     */
    @Override
    public PageResult<FaultDiagnosisVo> selectPage(FaultDiagnosisBo faultDiagnosisBo) {
        QueryWrapper queryWrapper = buildQueryWrapper(faultDiagnosisBo);
        Page<FaultDiagnosisVo> page = this.pageAs(PageQuery.build(), queryWrapper, FaultDiagnosisVo.class);
        return PageResult.build(page);
    }

    /**
     * 新增故障诊断
     *
     * @param faultDiagnosisBo 故障诊断Bo
     * @return 结果:true 操作成功，false 操作失败
     */
    @Override
    public FaultDiagnosis insert(FaultDiagnosisBo faultDiagnosisBo) {
        FaultDiagnosis faultDiagnosis = MapstructUtils.convert(faultDiagnosisBo, FaultDiagnosis.class);

        this.save(faultDiagnosis);//使用全局配置的雪花算法主键生成器生成ID值
        return faultDiagnosis;//使用全局配置的雪花算法主键生成器生成ID值
    }

    /**
     * 新增故障诊断，前台提供主键值，一般用于导入的场合
     *
     * @param faultDiagnosisBo 故障诊断Bo
     * @return 结果:true 操作成功，false 操作失败
     */
    @Override
    public boolean insertWithPk(FaultDiagnosisBo faultDiagnosisBo) {
        FaultDiagnosis faultDiagnosis = MapstructUtils.convert(faultDiagnosisBo, FaultDiagnosis.class);


        return faultDiagnosisMapper.insertWithPk(faultDiagnosis) > 0;//前台传来主键值
    }

    /**
     * 修改故障诊断
     *
     * @param faultDiagnosisBo 故障诊断Bo
     * @return 结果:true 更新成功，false 更新失败
     */
    @Override
    public boolean update(FaultDiagnosisBo faultDiagnosisBo) {
        FaultDiagnosis faultDiagnosis = MapstructUtils.convert(faultDiagnosisBo, FaultDiagnosis.class);
        if (ObjectUtil.isNotNull(faultDiagnosis) && ObjectUtil.isNotNull(faultDiagnosis.getId())) {
            return this.updateById(faultDiagnosis);
        }
        return false;
    }

    @Override
    public void asyncExport(List<FaultDiagnosisVo> listVo, String sheetName, LoginUser loginUser) {
        ExcelResultRes result = excelService.exportExcel(listVo, sheetName, FaultDiagnosisVo.class);
        boolean flag = importExportService.saveInfo(result, loginUser, "1");
        if (flag) {
            log.info("异步导出日志写入成功");
        }
    }

    /**
     * 批量删除故障诊断
     *
     * @param ids 需要删除的故障诊断主键集合
     * @return 结果:true 删除成功，false 删除失败
     */
    @Transactional
    @Override
    public boolean deleteByIds(Long[] ids) {
        return this.removeByIds(Arrays.asList(ids));
    }

}
