package org.eco.als.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.mybatisflex.core.paginate.Page;
import com.mybatisflex.core.query.QueryWrapper;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.eco.als.domain.Model;
import org.eco.als.domain.bo.ModelBo;
import org.eco.als.domain.vo.ModelImportVo;
import org.eco.als.domain.vo.ModelVo;
import org.eco.als.listener.ModelImportListener;
import org.eco.als.mapper.ModelMapper;
import org.eco.als.service.IModelService;
import org.eco.common.core.core.domain.model.LoginUser;
import org.eco.common.core.core.page.PageResult;
import org.eco.common.core.utils.MapstructUtils;
import org.eco.common.excel.entity.ExcelResultRes;
import org.eco.common.excel.service.IExcelService;
import org.eco.common.orm.core.page.PageQuery;
import org.eco.common.orm.core.service.impl.BaseServiceImpl;
import org.eco.system.service.IImportExportService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.Arrays;
import java.util.List;

import static org.eco.als.domain.table.ModelTableDef.MODEL;

/**
 * 模型信息Service业务层处理
 *
 * @author wgk
 * @date 2024-07-22
 */
@Service
@Slf4j
public class ModelServiceImpl extends BaseServiceImpl<ModelMapper, Model> implements IModelService {
    @Resource
    private ModelMapper modelMapper;

    @Resource
    private IExcelService excelService;

    @Resource
    private IImportExportService importExportService;

    @Override
    public QueryWrapper query() {
        return super.query().from(MODEL);
    }

    private QueryWrapper buildQueryWrapper(ModelBo modelBo) {
        QueryWrapper queryWrapper = super.buildBaseQueryWrapper();
        queryWrapper.and(MODEL.NAME.like
            (modelBo.getName()));
        queryWrapper.and(MODEL.TYPE.eq
            (modelBo.getType()));
        queryWrapper.and(MODEL.PART_ID.eq
            (modelBo.getPartId()));
        queryWrapper.and(MODEL.AIRCRAFT_TYPE.eq
            (modelBo.getAircraftType()));
        queryWrapper.orderBy(MODEL.UPDATE_TIME.desc());
        return queryWrapper;
    }

    /**
     * 查询模型信息
     *
     * @param id 模型信息主键
     * @return 模型信息
     */
    @Override
    public ModelVo selectById(Long id) {
        return this.getOneAs(query().where(MODEL.ID.eq(id)), ModelVo.class);

    }

    @Override
    public ModelVo selectByPartId(String partId) {
        return this.getOneAs(query().where(MODEL.PART_ID.eq(partId)), ModelVo.class);
    }


    /**
     * 查询模型信息列表
     *
     * @param modelBo 模型信息Bo
     * @return 模型信息集合
     */
    @Override
    public List<ModelVo> selectList(ModelBo modelBo) {
        QueryWrapper queryWrapper = buildQueryWrapper(modelBo);
        return this.listAs(queryWrapper, ModelVo.class);
    }

    /**
     * 分页查询模型信息列表
     *
     * @param modelBo 模型信息Bo
     * @return 分页模型信息集合
     */
    @Override
    public PageResult<ModelVo> selectPage(ModelBo modelBo) {
        QueryWrapper queryWrapper = buildQueryWrapper(modelBo);
        Page<ModelVo> page = this.pageAs(PageQuery.build(), queryWrapper, ModelVo.class);
        return PageResult.build(page);
    }

    /**
     * 新增模型信息
     *
     * @param modelBo 模型信息Bo
     * @return 结果:true 操作成功，false 操作失败
     */
    @Override
    public boolean insert(ModelBo modelBo) {
        Model model = MapstructUtils.convert(modelBo, Model.class);

        return this.save(model);//使用全局配置的雪花算法主键生成器生成ID值
    }

    /**
     * 新增模型信息，前台提供主键值，一般用于导入的场合
     *
     * @param modelBo 模型信息Bo
     * @return 结果:true 操作成功，false 操作失败
     */
    @Override
    public boolean insertWithPk(ModelBo modelBo) {
        Model model = MapstructUtils.convert(modelBo, Model.class);


        return modelMapper.insertWithPk(model) > 0;//前台传来主键值
    }

    /**
     * 修改模型信息
     *
     * @param modelBo 模型信息Bo
     * @return 结果:true 更新成功，false 更新失败
     */
    @Override
    public boolean update(ModelBo modelBo) {
        Model model = MapstructUtils.convert(modelBo, Model.class);
        if (ObjectUtil.isNotNull(model) && ObjectUtil.isNotNull(model.getId())) {
            return this.updateById(model);
        }
        return false;
    }

    @Override
    public void asyncImportData(MultipartFile file, boolean updateSupport, LoginUser loginUser) {
        ExcelResultRes result;
        try {
            String name = file.getOriginalFilename();
            result = excelService.importExcel(file.getInputStream(), name, ModelImportVo.class, new ModelImportListener(updateSupport, loginUser));
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        boolean flag = importExportService.saveInfo(result, loginUser, "0");
        if (flag) {
            log.info("异步导入日志写入成功");
        }
    }

    @Override
    public void asyncExport(List<ModelVo> listVo, String sheetName, LoginUser loginUser) {
        ExcelResultRes result = excelService.exportExcel(listVo, sheetName, ModelVo.class);
        boolean flag = importExportService.saveInfo(result, loginUser, "1");
        if (flag) {
            log.info("异步导出日志写入成功");
        }
    }

    /**
     * 批量删除模型信息
     *
     * @param ids 需要删除的模型信息主键集合
     * @return 结果:true 删除成功，false 删除失败
     */
    @Transactional
    @Override
    public boolean deleteByIds(Long[] ids) {
        return this.removeByIds(Arrays.asList(ids));
    }

}
