package org.eco.als.service.impl;

import java.util.Arrays;
import java.util.List;

import cn.hutool.core.util.ObjectUtil;
import com.mybatisflex.core.paginate.Page;
import com.mybatisflex.core.query.QueryWrapper;
import lombok.extern.slf4j.Slf4j;
import org.eco.common.core.core.domain.model.LoginUser;
import org.eco.common.excel.entity.ExcelResultRes;
import org.eco.common.excel.service.IExcelService;
import org.eco.common.core.utils.bean.BeanUtils;
import org.eco.common.core.utils.MapstructUtils;
import org.eco.system.service.IImportExportService;
import org.eco.common.core.utils.StringUtils;
import org.eco.system.domain.bo.ImportExportBo;
import org.eco.common.orm.core.page.PageQuery;
import org.eco.common.core.core.page.PageResult;
import org.eco.common.orm.core.service.impl.BaseServiceImpl;
import jakarta.annotation.Resource;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;

import org.eco.als.mapper.PersonnelAllocationMapper;
import org.eco.als.domain.PersonnelAllocation;
import org.eco.als.domain.bo.PersonnelAllocationBo;
import org.eco.als.domain.vo.PersonnelAllocationVo;
import org.eco.als.domain.vo.PersonnelAllocationImportVo;
import org.eco.als.listener.PersonnelAllocationImportListener;
import org.eco.als.service.IPersonnelAllocationService;

import static org.eco.als.domain.table.PersonnelAllocationTableDef.PERSONNEL_ALLOCATION;

/**
 * 维修任务任务派遣Service业务层处理
 *
 * @author wgk
 * @date 2024-11-14
 */
@Service
@Slf4j
public class PersonnelAllocationServiceImpl extends BaseServiceImpl<PersonnelAllocationMapper, PersonnelAllocation> implements IPersonnelAllocationService {
    @Resource
    private PersonnelAllocationMapper personnelAllocationMapper;

    @Resource
    private IExcelService excelService;

    @Resource
    private IImportExportService importExportService;

    @Override
    public QueryWrapper query() {
        return super.query().from(PERSONNEL_ALLOCATION);
    }

    private QueryWrapper buildQueryWrapper(PersonnelAllocationBo personnelAllocationBo) {
        QueryWrapper queryWrapper = super.buildBaseQueryWrapper();
        queryWrapper.and(PERSONNEL_ALLOCATION.TASK_ID.eq
            (personnelAllocationBo.getTaskId()));
        queryWrapper.and(PERSONNEL_ALLOCATION.PERSONNEL.eq
            (personnelAllocationBo.getPersonnel()));
        queryWrapper.and(PERSONNEL_ALLOCATION.POSITION.eq
            (personnelAllocationBo.getPosition()));
        queryWrapper.and(PERSONNEL_ALLOCATION.QUANTITY.eq
            (personnelAllocationBo.getQuantity()));
        queryWrapper.and(PERSONNEL_ALLOCATION.DUTIES.eq
            (personnelAllocationBo.getDuties()));
        queryWrapper.and(PERSONNEL_ALLOCATION.REMARKS.eq
            (personnelAllocationBo.getRemarks()));

        return queryWrapper;
    }

    /**
     * 查询维修任务任务派遣
     *
     * @param id 维修任务任务派遣主键
     * @return 维修任务任务派遣
     */
    @Override
    public PersonnelAllocationVo selectById(Long id) {
        return this.getOneAs(query().where(PERSONNEL_ALLOCATION.ID.eq(id)), PersonnelAllocationVo.class);

    }


    /**
     * 查询维修任务任务派遣列表
     *
     * @param personnelAllocationBo 维修任务任务派遣Bo
     * @return 维修任务任务派遣集合
     */
    @Override
    public List<PersonnelAllocationVo> selectList(PersonnelAllocationBo personnelAllocationBo) {
        QueryWrapper queryWrapper = buildQueryWrapper(personnelAllocationBo);
        return this.listAs(queryWrapper, PersonnelAllocationVo.class);
    }

    /**
     * 分页查询维修任务任务派遣列表
     *
     * @param personnelAllocationBo 维修任务任务派遣Bo
     * @return 分页维修任务任务派遣集合
     */
    @Override
    public PageResult<PersonnelAllocationVo> selectPage(PersonnelAllocationBo personnelAllocationBo) {
        QueryWrapper queryWrapper = buildQueryWrapper(personnelAllocationBo);
        Page<PersonnelAllocationVo> page = this.pageAs(PageQuery.build(), queryWrapper, PersonnelAllocationVo.class);
        return PageResult.build(page);
    }

    /**
     * 新增维修任务任务派遣
     *
     * @param personnelAllocationBo 维修任务任务派遣Bo
     * @return 结果:true 操作成功，false 操作失败
     */
    @Override
    public boolean insert(PersonnelAllocationBo personnelAllocationBo) {
        PersonnelAllocation personnelAllocation = MapstructUtils.convert(personnelAllocationBo, PersonnelAllocation.class);

        return this.save(personnelAllocation);//使用全局配置的雪花算法主键生成器生成ID值
    }

    /**
     * 新增维修任务任务派遣，前台提供主键值，一般用于导入的场合
     *
     * @param personnelAllocationBo 维修任务任务派遣Bo
     * @return 结果:true 操作成功，false 操作失败
     */
    @Override
    public boolean insertWithPk(PersonnelAllocationBo personnelAllocationBo) {
        PersonnelAllocation personnelAllocation = MapstructUtils.convert(personnelAllocationBo, PersonnelAllocation.class);


        return personnelAllocationMapper.insertWithPk(personnelAllocation) > 0;//前台传来主键值
    }

    /**
     * 修改维修任务任务派遣
     *
     * @param personnelAllocationBo 维修任务任务派遣Bo
     * @return 结果:true 更新成功，false 更新失败
     */
    @Override
    public boolean update(PersonnelAllocationBo personnelAllocationBo) {
        PersonnelAllocation personnelAllocation = MapstructUtils.convert(personnelAllocationBo, PersonnelAllocation.class);
        if (ObjectUtil.isNotNull(personnelAllocation) && ObjectUtil.isNotNull(personnelAllocation.getId())) {
            boolean updated = this.updateById(personnelAllocation);
            return updated;
        }
        return false;
    }

    @Override
    public void asyncImportData(MultipartFile file, boolean updateSupport, LoginUser loginUser) {
        ExcelResultRes result;
        try {
            String name = file.getOriginalFilename();
            result = excelService.importExcel(file.getInputStream(), name, PersonnelAllocationImportVo.class, new PersonnelAllocationImportListener(updateSupport, loginUser));
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        boolean flag = importExportService.saveInfo(result, loginUser, "0");
        if (flag) {
            log.info("异步导入日志写入成功");
        }
    }

    @Override
    public void asyncExport(List<PersonnelAllocationVo> listVo, String sheetName, LoginUser loginUser) {
        ExcelResultRes result = excelService.exportExcel(listVo, sheetName, PersonnelAllocationVo.class);
        boolean flag = importExportService.saveInfo(result, loginUser, "1");
        if (flag) {
            log.info("异步导出日志写入成功");
        }
    }

    /**
     * 批量删除维修任务任务派遣
     *
     * @param ids 需要删除的维修任务任务派遣主键集合
     * @return 结果:true 删除成功，false 删除失败
     */
    @Transactional
    @Override
    public boolean deleteByIds(Long[] ids) {
        return this.removeByIds(Arrays.asList(ids));
    }

}
