package org.eco.als.service.impl;

import java.util.Arrays;
import java.util.List;

import cn.hutool.core.util.ObjectUtil;
import com.mybatisflex.core.paginate.Page;
import com.mybatisflex.core.query.QueryWrapper;
import lombok.extern.slf4j.Slf4j;
import org.eco.common.core.core.domain.model.LoginUser;
import org.eco.common.excel.entity.ExcelResultRes;
import org.eco.common.excel.service.IExcelService;
import org.eco.common.core.utils.bean.BeanUtils;
import org.eco.common.core.utils.MapstructUtils;
import org.eco.system.service.IImportExportService;
import org.eco.common.core.utils.StringUtils;
import org.eco.system.domain.bo.ImportExportBo;
import org.eco.common.orm.core.page.PageQuery;
import org.eco.common.core.core.page.PageResult;
import org.eco.common.orm.core.service.impl.BaseServiceImpl;
import jakarta.annotation.Resource;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;

import org.eco.als.mapper.ResourceAllocationMapper;
import org.eco.als.domain.ResourceAllocation;
import org.eco.als.domain.bo.ResourceAllocationBo;
import org.eco.als.domain.vo.ResourceAllocationVo;
import org.eco.als.domain.vo.ResourceAllocationImportVo;
import org.eco.als.listener.ResourceAllocationImportListener;
import org.eco.als.service.IResourceAllocationService;

import static org.eco.als.domain.table.ResourceAllocationTableDef.RESOURCE_ALLOCATION;

/**
 * 维修任务资源分配Service业务层处理
 *
 * @author wgk
 * @date 2024-11-14
 */
@Service
@Slf4j
public class ResourceAllocationServiceImpl extends BaseServiceImpl<ResourceAllocationMapper, ResourceAllocation> implements IResourceAllocationService {
    @Resource
    private ResourceAllocationMapper resourceAllocationMapper;

    @Resource
    private IExcelService excelService;

    @Resource
    private IImportExportService importExportService;

    @Override
    public QueryWrapper query() {
        return super.query().from(RESOURCE_ALLOCATION);
    }

    private QueryWrapper buildQueryWrapper(ResourceAllocationBo resourceAllocationBo) {
        QueryWrapper queryWrapper = super.buildBaseQueryWrapper();
        queryWrapper.and(RESOURCE_ALLOCATION.TASK_ID.eq
            (resourceAllocationBo.getTaskId()));
        queryWrapper.and(RESOURCE_ALLOCATION.DEVICE_CODE.eq
            (resourceAllocationBo.getDeviceCode()));
        queryWrapper.and(RESOURCE_ALLOCATION.DEVICE_NAME.like
            (resourceAllocationBo.getDeviceName()));
        queryWrapper.and(RESOURCE_ALLOCATION.TYPE.eq
            (resourceAllocationBo.getType()));
        queryWrapper.and(RESOURCE_ALLOCATION.MODEL_NUMBER.eq
            (resourceAllocationBo.getModelNumber()));
        queryWrapper.and(RESOURCE_ALLOCATION.QUANTITY.eq
            (resourceAllocationBo.getQuantity()));
        queryWrapper.and(RESOURCE_ALLOCATION.PURPOSE.eq
            (resourceAllocationBo.getPurpose()));
        queryWrapper.and(RESOURCE_ALLOCATION.REMARKS.eq
            (resourceAllocationBo.getRemarks()));

        return queryWrapper;
    }

    /**
     * 查询维修任务资源分配
     *
     * @param id 维修任务资源分配主键
     * @return 维修任务资源分配
     */
    @Override
    public ResourceAllocationVo selectById(Long id) {
        return this.getOneAs(query().where(RESOURCE_ALLOCATION.ID.eq(id)), ResourceAllocationVo.class);

    }


    /**
     * 查询维修任务资源分配列表
     *
     * @param resourceAllocationBo 维修任务资源分配Bo
     * @return 维修任务资源分配集合
     */
    @Override
    public List<ResourceAllocationVo> selectList(ResourceAllocationBo resourceAllocationBo) {
        QueryWrapper queryWrapper = buildQueryWrapper(resourceAllocationBo);
        return this.listAs(queryWrapper, ResourceAllocationVo.class);
    }

    /**
     * 分页查询维修任务资源分配列表
     *
     * @param resourceAllocationBo 维修任务资源分配Bo
     * @return 分页维修任务资源分配集合
     */
    @Override
    public PageResult<ResourceAllocationVo> selectPage(ResourceAllocationBo resourceAllocationBo) {
        QueryWrapper queryWrapper = buildQueryWrapper(resourceAllocationBo);
        Page<ResourceAllocationVo> page = this.pageAs(PageQuery.build(), queryWrapper, ResourceAllocationVo.class);
        return PageResult.build(page);
    }

    /**
     * 新增维修任务资源分配
     *
     * @param resourceAllocationBo 维修任务资源分配Bo
     * @return 结果:true 操作成功，false 操作失败
     */
    @Override
    public boolean insert(ResourceAllocationBo resourceAllocationBo) {
        ResourceAllocation resourceAllocation = MapstructUtils.convert(resourceAllocationBo, ResourceAllocation.class);

        return this.save(resourceAllocation);//使用全局配置的雪花算法主键生成器生成ID值
    }

    /**
     * 新增维修任务资源分配，前台提供主键值，一般用于导入的场合
     *
     * @param resourceAllocationBo 维修任务资源分配Bo
     * @return 结果:true 操作成功，false 操作失败
     */
    @Override
    public boolean insertWithPk(ResourceAllocationBo resourceAllocationBo) {
        ResourceAllocation resourceAllocation = MapstructUtils.convert(resourceAllocationBo, ResourceAllocation.class);


        return resourceAllocationMapper.insertWithPk(resourceAllocation) > 0;//前台传来主键值
    }

    /**
     * 修改维修任务资源分配
     *
     * @param resourceAllocationBo 维修任务资源分配Bo
     * @return 结果:true 更新成功，false 更新失败
     */
    @Override
    public boolean update(ResourceAllocationBo resourceAllocationBo) {
        ResourceAllocation resourceAllocation = MapstructUtils.convert(resourceAllocationBo, ResourceAllocation.class);
        if (ObjectUtil.isNotNull(resourceAllocation) && ObjectUtil.isNotNull(resourceAllocation.getId())) {
            boolean updated = this.updateById(resourceAllocation);
            return updated;
        }
        return false;
    }

    @Override
    public void asyncImportData(MultipartFile file, boolean updateSupport, LoginUser loginUser) {
        ExcelResultRes result;
        try {
            String name = file.getOriginalFilename();
            result = excelService.importExcel(file.getInputStream(), name, ResourceAllocationImportVo.class, new ResourceAllocationImportListener(updateSupport, loginUser));
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        boolean flag = importExportService.saveInfo(result, loginUser, "0");
        if (flag) {
            log.info("异步导入日志写入成功");
        }
    }

    @Override
    public void asyncExport(List<ResourceAllocationVo> listVo, String sheetName, LoginUser loginUser) {
        ExcelResultRes result = excelService.exportExcel(listVo, sheetName, ResourceAllocationVo.class);
        boolean flag = importExportService.saveInfo(result, loginUser, "1");
        if (flag) {
            log.info("异步导出日志写入成功");
        }
    }

    /**
     * 批量删除维修任务资源分配
     *
     * @param ids 需要删除的维修任务资源分配主键集合
     * @return 结果:true 删除成功，false 删除失败
     */
    @Transactional
    @Override
    public boolean deleteByIds(Long[] ids) {
        return this.removeByIds(Arrays.asList(ids));
    }

}
