package org.eco.als.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.mybatisflex.core.paginate.Page;
import com.mybatisflex.core.query.QueryWrapper;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.eco.als.domain.Sortie;
import org.eco.als.domain.bo.SortieBo;
import org.eco.als.domain.vo.SortieImportVo;
import org.eco.als.domain.vo.SortieVo;
import org.eco.als.listener.SortieImportListener;
import org.eco.als.mapper.SortieMapper;
import org.eco.als.service.ISortieService;
import org.eco.common.core.core.domain.model.LoginUser;
import org.eco.common.core.core.page.PageResult;
import org.eco.common.core.utils.MapstructUtils;
import org.eco.common.core.utils.bean.BeanUtils;
import org.eco.common.excel.entity.ExcelResultRes;
import org.eco.common.excel.service.IExcelService;
import org.eco.common.orm.core.page.PageQuery;
import org.eco.common.orm.core.service.impl.BaseServiceImpl;
import org.eco.system.domain.bo.ImportExportBo;
import org.eco.system.service.IImportExportService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.Arrays;
import java.util.List;

import static org.eco.als.domain.table.SortieTableDef.SORTIE;

/**
 * 架次Service业务层处理
 *
 * @author wgk
 * @date 2024-07-18
 */
@Service
@Slf4j
public class SortieServiceImpl extends BaseServiceImpl<SortieMapper, Sortie> implements ISortieService {
    @Resource
    private SortieMapper sortieMapper;

    @Resource
    private IExcelService excelService;

    @Resource
    private IImportExportService importExportService;

    @Override
    public QueryWrapper query() {
        return super.query().from(SORTIE);
    }

    private QueryWrapper buildQueryWrapper(SortieBo sortieBo) {
        QueryWrapper queryWrapper = super.buildBaseQueryWrapper();
        queryWrapper.and(SORTIE.SORTIE_NO.eq
            (sortieBo.getSortieNo()));
        queryWrapper.and(SORTIE.AIRCRAFT_ID.eq
            (sortieBo.getAircraftId()));
        queryWrapper.and(SORTIE.FLIGHT_DATE.eq
            (sortieBo.getFlightDate()));

        return queryWrapper;
    }

    /**
     * 查询架次
     *
     * @param id 架次主键
     * @return 架次
     */
    @Override
    public SortieVo selectById(Long id) {
        return this.getOneAs(query().where(SORTIE.ID.eq(id)), SortieVo.class);

    }

    /**
     * 查询架次列表
     *
     * @param sortieBo 架次Bo
     * @return 架次集合
     */
    @Override
    public List<SortieVo> selectList(SortieBo sortieBo) {
        QueryWrapper queryWrapper = buildQueryWrapper(sortieBo);
        return this.listAs(queryWrapper, SortieVo.class);
    }

    /**
     * 分页查询架次列表
     *
     * @param sortieBo 架次Bo
     * @return 分页架次集合
     */
    @Override
    public PageResult<SortieVo> selectPage(SortieBo sortieBo) {
        QueryWrapper queryWrapper = buildQueryWrapper(sortieBo);
        Page<SortieVo> page = this.pageAs(PageQuery.build(), queryWrapper, SortieVo.class);
        return PageResult.build(page);
    }

    /**
     * 新增架次
     *
     * @param sortieBo 架次Bo
     * @return 结果:true 操作成功，false 操作失败
     */
    @Override
    public boolean insert(SortieBo sortieBo) {
        Sortie sortie = MapstructUtils.convert(sortieBo, Sortie.class);

        return this.save(sortie);//使用全局配置的雪花算法主键生成器生成ID值
    }

    /**
     * 新增架次，前台提供主键值，一般用于导入的场合
     *
     * @param sortieBo 架次Bo
     * @return 结果:true 操作成功，false 操作失败
     */
    @Override
    public boolean insertWithPk(SortieBo sortieBo) {
        Sortie sortie = MapstructUtils.convert(sortieBo, Sortie.class);


        return sortieMapper.insertWithPk(sortie) > 0;//前台传来主键值
    }

    /**
     * 修改架次
     *
     * @param sortieBo 架次Bo
     * @return 结果:true 更新成功，false 更新失败
     */
    @Override
    public boolean update(SortieBo sortieBo) {
        Sortie sortie = MapstructUtils.convert(sortieBo, Sortie.class);
        if (ObjectUtil.isNotNull(sortie) && ObjectUtil.isNotNull(sortie.getId())) {
            return this.updateById(sortie);
        }
        return false;
    }

    @Override
    public void asyncImportData(MultipartFile file, boolean updateSupport, LoginUser loginUser) {
        ExcelResultRes result;
        try {
            String name = file.getOriginalFilename();
            result = excelService.importExcel(file.getInputStream(), name, SortieImportVo.class, new SortieImportListener(updateSupport, loginUser));
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        ImportExportBo bo = new ImportExportBo();
        BeanUtils.copyProperties(result, bo);
        bo.setUpdateBy(loginUser.getUserId());
        bo.setCreateBy(loginUser.getUserId());
        bo.setType("0");
        boolean flag = importExportService.insert(bo);
        if (flag) {
            log.info("异步导入日志写入成功");
        }
    }

    @Override
    public void asyncExport(List<SortieVo> listVo, String sheetName, LoginUser loginUser) {
        ExcelResultRes result = excelService.exportExcel(listVo, sheetName, SortieVo.class);
        ImportExportBo bo = new ImportExportBo();
        BeanUtils.copyProperties(result, bo);
        bo.setUpdateBy(loginUser.getUserId());
        bo.setCreateBy(loginUser.getUserId());
        bo.setType("1");
        boolean flag = importExportService.insert(bo);
        if (flag) {
            log.info("异步导出日志写入成功");
        }
    }

    /**
     * 批量删除架次
     *
     * @param ids 需要删除的架次主键集合
     * @return 结果:true 删除成功，false 删除失败
     */
    @Transactional
    @Override
    public boolean deleteByIds(Long[] ids) {
        return this.removeByIds(Arrays.asList(ids));
    }

}
