package org.eco.als.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.mybatisflex.core.paginate.Page;
import com.mybatisflex.core.query.QueryWrapper;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.eco.als.domain.Warning;
import org.eco.als.domain.bo.WarningBo;
import org.eco.als.domain.vo.WarningVo;
import org.eco.als.mapper.WarningMapper;
import org.eco.als.service.IWarningService;
import org.eco.common.core.core.page.PageResult;
import org.eco.common.core.utils.MapstructUtils;
import org.eco.common.core.utils.StringUtils;
import org.eco.common.excel.service.IExcelService;
import org.eco.common.orm.core.page.PageQuery;
import org.eco.common.orm.core.service.impl.BaseServiceImpl;
import org.eco.system.service.IImportExportService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Arrays;
import java.util.List;

import static org.eco.als.domain.table.WarningTableDef.WARNING;

/**
 * 警告信息Service业务层处理
 *
 * @author wgk
 * @date 2024-07-22
 */
@Service
@Slf4j
public class WarningServiceImpl extends BaseServiceImpl<WarningMapper, Warning> implements IWarningService {
    @Resource
    private WarningMapper warningMapper;

    @Resource
    private IExcelService excelService;

    @Resource
    private IImportExportService importExportService;

    @Override
    public QueryWrapper query() {
        return super.query().from(WARNING);
    }

    private QueryWrapper buildQueryWrapper(WarningBo warningBo) {
        QueryWrapper queryWrapper = super.buildBaseQueryWrapper();
        queryWrapper.and(WARNING.SORTIE_NO.eq
            (warningBo.getSortieNo()));
        if (StringUtils.isNotBlank(warningBo.getAircraftId())) {
            queryWrapper.and(WARNING.AIRCRAFT_ID.in
                (warningBo.getAircraftId().split(",")));
        }
        queryWrapper.and(WARNING.CODE.eq
            (warningBo.getCode()));
        queryWrapper.and(WARNING.SORTIE_NO.eq
            (warningBo.getSortieNo()));
        queryWrapper.and(WARNING.NAME.like
            (warningBo.getName()));
        queryWrapper.and(WARNING.DESCRIBE.eq
            (warningBo.getDescribe()));

        return queryWrapper;
    }

    /**
     * 查询警告信息
     *
     * @param id 警告信息主键
     * @return 警告信息
     */
    @Override
    public WarningVo selectById(Long id) {
        return this.getOneAs(query().where(WARNING.ID.eq(id)), WarningVo.class);

    }


    /**
     * 查询警告信息列表
     *
     * @param warningBo 警告信息Bo
     * @return 警告信息集合
     */
    @Override
    public List<WarningVo> selectList(WarningBo warningBo) {
        QueryWrapper queryWrapper = buildQueryWrapper(warningBo);
        return this.listAs(queryWrapper, WarningVo.class);
    }

    /**
     * 分页查询警告信息列表
     *
     * @param warningBo 警告信息Bo
     * @return 分页警告信息集合
     */
    @Override
    public PageResult<WarningVo> selectPage(WarningBo warningBo) {
        QueryWrapper queryWrapper = buildQueryWrapper(warningBo);
        Page<WarningVo> page = this.pageAs(PageQuery.build(), queryWrapper, WarningVo.class);
        return PageResult.build(page);
    }

    /**
     * 新增警告信息
     *
     * @param warningBo 警告信息Bo
     * @return 结果:true 操作成功，false 操作失败
     */
    @Override
    public boolean insert(WarningBo warningBo) {
        Warning warning = MapstructUtils.convert(warningBo, Warning.class);

        return this.save(warning);//使用全局配置的雪花算法主键生成器生成ID值
    }

    /**
     * 新增警告信息，前台提供主键值，一般用于导入的场合
     *
     * @param warningBo 警告信息Bo
     * @return 结果:true 操作成功，false 操作失败
     */
    @Override
    public boolean insertWithPk(WarningBo warningBo) {
        Warning warning = MapstructUtils.convert(warningBo, Warning.class);


        return warningMapper.insertWithPk(warning) > 0;//前台传来主键值
    }

    /**
     * 修改警告信息
     *
     * @param warningBo 警告信息Bo
     * @return 结果:true 更新成功，false 更新失败
     */
    @Override
    public boolean update(WarningBo warningBo) {
        Warning warning = MapstructUtils.convert(warningBo, Warning.class);
        if (ObjectUtil.isNotNull(warning) && ObjectUtil.isNotNull(warning.getId())) {
            return this.updateById(warning);
        }
        return false;
    }

    /**
     * 批量删除警告信息
     *
     * @param ids 需要删除的警告信息主键集合
     * @return 结果:true 删除成功，false 删除失败
     */
    @Transactional
    @Override
    public boolean deleteByIds(Long[] ids) {
        return this.removeByIds(Arrays.asList(ids));
    }

}
