package org.eco.web.service.impl;

import cn.dev33.satoken.stp.SaLoginModel;
import cn.dev33.satoken.stp.StpUtil;
import cn.hutool.core.util.ObjectUtil;
import org.eco.web.domain.vo.LoginVo;
import org.eco.web.service.IAuthStrategy;
import org.eco.web.service.SysLoginService;
import org.eco.common.core.constant.Constants;
import org.eco.common.core.constant.GlobalConstants;
import org.eco.common.core.core.domain.model.EmailLoginBody;
import org.eco.common.core.core.domain.model.LoginUser;
import org.eco.common.core.enums.LoginType;
import org.eco.common.core.enums.UserStatus;
import org.eco.common.core.exception.user.CaptchaExpireException;
import org.eco.common.core.exception.user.UserException;
import org.eco.common.core.utils.MessageUtils;
import org.eco.common.core.utils.StringUtils;
import org.eco.common.core.utils.ValidatorUtils;
import org.eco.common.json.utils.JsonUtils;
import org.eco.common.redis.utils.RedisUtils;
import org.eco.common.security.utils.LoginHelper;
import org.eco.common.tenant.helper.TenantHelper;
import org.eco.system.domain.SysClient;
import org.eco.system.domain.vo.SysUserVo;
import org.eco.system.service.ISysUserService;
import jakarta.annotation.Resource;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

/**
 * 邮件认证策略
 *
 * @author wgk
 */
@Slf4j
@Service("email" + IAuthStrategy.BASE_NAME)
@RequiredArgsConstructor
public class EmailAuthStrategy implements IAuthStrategy {

    @Resource
    private final SysLoginService loginService;
    @Resource
    private ISysUserService userService;

    @Override
    public LoginVo login(String body, SysClient client) {
        EmailLoginBody loginBody = JsonUtils.parseObject(body, EmailLoginBody.class);
        ValidatorUtils.validate(loginBody);
        Long tenantId = loginBody.getTenantId();
        String email = loginBody.getEmail();
        String emailCode = loginBody.getEmailCode();

        // 通过邮箱查找用户
        SysUserVo user = loadUserByEmail(tenantId, email);

        loginService.checkLogin(LoginType.EMAIL, tenantId, user.getUserName(), () -> !validateEmailCode(tenantId, email, emailCode));
        // 此处可根据登录用户的数据不同 自行创建 loginUser 属性不够用继承扩展就行了
        LoginUser loginUser = loginService.buildLoginUser(user);
        loginUser.setClientKey(client.getClientKey());
        loginUser.setDeviceType(client.getDeviceType());
        SaLoginModel model = new SaLoginModel();
        model.setDevice(client.getDeviceType());
        // 自定义分配 不同用户体系 不同 token 授权时间 不设置默认走全局 yml 配置
        // 例如: 后台用户30分钟过期 app用户1天过期
        model.setTimeout(client.getTimeout());
        model.setActiveTimeout(client.getActiveTimeout());
        model.setExtra(LoginHelper.CLIENT_KEY, client.getClientId());
        // 生成token
        LoginHelper.login(loginUser, model);

//        loginService.recordLogininfor(loginUser.getTenantId(), user.getUserName(), Constants.LOGIN_SUCCESS, MessageUtils.message("user.login.success"));
//        loginService.recordLoginInfo(user.getUserId());

        LoginVo loginVo = new LoginVo();
        loginVo.setAccessToken(StpUtil.getTokenValue());
        loginVo.setExpireIn(StpUtil.getTokenTimeout());
        loginVo.setClientId(client.getClientId());
        return loginVo;
    }

    /**
     * 校验邮箱验证码
     */
    private boolean validateEmailCode(Long tenantId, String email, String emailCode) {
        String code = RedisUtils.getCacheObject(GlobalConstants.CAPTCHA_CODE_KEY + email);
        if (StringUtils.isBlank(code)) {
            loginService.recordLogininfor(tenantId, email, Constants.LOGIN_FAIL, MessageUtils.message("user.jcaptcha.expire"));
            throw new CaptchaExpireException();
        }
        return code.equals(emailCode);
    }

    private SysUserVo loadUserByEmail(Long tenantId, String email) {
        return TenantHelper.dynamic(tenantId, () -> {
            SysUserVo user = userService.selectUserByEmail(email);
            if (ObjectUtil.isNull(user)) {
                log.info("登录用户：{} 不存在.", email);
                throw new UserException("user.not.exists", email);
            } else if (UserStatus.DISABLE.getCode().equals(user.getStatus())) {
                log.info("登录用户：{} 已被停用.", email);
                throw new UserException("user.blocked", email);
            }
            return user;
        });

    }

}
